@@ -53,13 +53,16 @@
 #include "gdm-session-direct.h"
 #include "gdm-greeter-server.h"
 #include "gdm-greeter-session.h"
+#include "gdm-settings-direct.h"
+#include "gdm-settings-keys.h"
 
 #define GDM_SIMPLE_SLAVE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_SIMPLE_SLAVE, GdmSimpleSlavePrivate))
 
 #define GDM_DBUS_NAME              "org.gnome.DisplayManager"
 #define GDM_DBUS_DISPLAY_INTERFACE "org.gnome.DisplayManager.Display"
 
-#define MAX_CONNECT_ATTEMPTS 10
+#define MAX_CONNECT_ATTEMPTS  10
+#define DEFAULT_PING_INTERVAL 15
 
 struct GdmSimpleSlavePrivate
 {
@@ -214,7 +217,7 @@ on_session_setup_failed (GdmSession     *session,
 {
         if (slave->priv->greeter_server != NULL) {
                 gdm_greeter_server_problem (slave->priv->greeter_server,
-                                            _("Unable to initialize login system"));
+                                           message != NULL ? message:  _("Unable to initialize login system"));
         }
 
         destroy_session (slave);
@@ -250,7 +253,7 @@ on_session_authentication_failed (GdmSession     *session,
 {
         if (slave->priv->greeter_server != NULL) {
                 gdm_greeter_server_problem (slave->priv->greeter_server,
-                                            _("Unable to authenticate user"));
+                                            message != NULL ? message : _("Unable to authenticate user"));
         }
         destroy_session (slave);
         queue_greeter_reset (slave);
@@ -304,7 +307,7 @@ on_session_authorization_failed (GdmSession     *session,
 {
         if (slave->priv->greeter_server != NULL) {
                 gdm_greeter_server_problem (slave->priv->greeter_server,
-                                            _("Unable to authorize user"));
+                                           message != NULL ? message :  _("Unable to authorize user"));
         }
 
         destroy_session (slave);
@@ -422,8 +425,14 @@ on_session_accreditation_failed (GdmSession     *session,
            accreditation fails */
         if (! migrated) {
                 if (slave->priv->greeter_server != NULL) {
+                        const char *problem;
+                        if (message) {
+                                problem = message;
+                        } else {
+                                problem = _("Unable to establish credentials");
+                        }
                         gdm_greeter_server_problem (slave->priv->greeter_server,
-                                                    _("Unable establish credentials"));
+                                                    problem);
                 }
         }
 
@@ -862,6 +871,7 @@ start_greeter (GdmSimpleSlave *slave)
         char          *display_hostname;
         char          *auth_file;
         char          *address;
+        gboolean       res;
 
         g_debug ("GdmSimpleSlave: Running greeter");
 
@@ -889,6 +899,10 @@ start_greeter (GdmSimpleSlave *slave)
         /* FIXME: send a signal back to the master */
 
         /* If XDMCP setup pinging */
+        slave->priv->ping_interval = DEFAULT_PING_INTERVAL;
+        res = gdm_settings_direct_get_int (GDM_KEY_PING_INTERVAL,
+                                           &(slave->priv->ping_interval));
+
         if ( ! display_is_local && slave->priv->ping_interval > 0) {
                 alarm (slave->priv->ping_interval);
         }
@@ -1025,7 +1039,7 @@ static void
 on_server_ready (GdmServer      *server,
                  GdmSimpleSlave *slave)
 {
-        g_timeout_add (500, (GSourceFunc)idle_connect_to_display, slave);
+        g_idle_add ((GSourceFunc)idle_connect_to_display, slave);
 }
 
 static void
