@@ -248,6 +248,12 @@ on_select_layout (GdmGreeterSession      *session,
                                                text);
 }
 
+static void
+on_dialog_hidden (GdmGreeterSession     *session)
+{
+        gtk_window_present (GTK_WINDOW (session->priv->login_window));
+}
+
 static void
 on_select_user (GdmGreeterLoginWindow *login_window,
                 const char            *text,
@@ -349,6 +355,11 @@ toggle_panel (GdmGreeterSession *session,
                                           G_CALLBACK (on_select_session),
                                           session);
 
+                g_signal_connect_swapped (session->priv->panel,
+                                          "dialog-hidden",
+                                          G_CALLBACK (on_dialog_hidden),
+                                          session);
+
                 gtk_widget_show (session->priv->panel);
         } else {
                 gtk_widget_destroy (session->priv->panel);
